
#include <stdio.h>
#include "splaytree.h"
#include "astring.h"

template <class T>
void
FreeTree(binaryNode<T> *aTree)
{
	if (aTree != 0)
	{
		// Make sure we're not recursively trying to 
		// delete the same tree because either the left
		// or right nodes are the same as the tree itself.
		if ((aTree->Left() != aTree))
		{
			FreeTree(aTree->Left());
			aTree->SetLeft(0);
		}
		
		if ((aTree->Right() != aTree))
		{
			FreeTree(aTree->Right());
			aTree->SetRight(0);
		}
			
		// Now that the children have been deleted, delete the
		// node itself.
		delete aTree;
	}

}

static void
TestNodes()
{
	binaryNode<long> *node = new binaryNode<long>;
	binaryNode<long> *lnode = new binaryNode<long>;
	binaryNode<long> *rnode = new binaryNode<long>;
	
	node->SetLeft(lnode);
	node->SetRight(rnode);
	
	binaryNode<long> *newNode(node);
	
	binaryNode<long> *circular = new binaryNode<long>;
	circular->SetLeft(circular);
	circular->SetRight(circular);
	
	node->SetLeft(circular);
	
	FreeTree(node);
	//delete circular;
	
	//FreeTree(circular);
	
	//delete circular;
	//delete node;

}

main()
{
	TestNodes();
	

	String name1("William");
	String name2("Albert");
	
	splayTree<String> *nametree = new splayTree<String>;
	nametree->Insert(name1);
	nametree->Insert(name2);

	splayTree<long> tree;
	
	tree.Insert(3);
	tree.Insert(1);
	tree.Insert(4);
	tree.Insert(5);
	tree.Insert(2);
	tree.Insert(9);
	tree.Insert(6);
	tree.Insert(8);
	
	// Check for emptiness
	printf("IsEmpty - %d\n", tree.IsEmpty());
	
	// See if find works
	printf("Includes (5) - %d\n", tree.Includes(5));
	printf("Includes (10) - %d\n", tree.Includes(10));
	printf("Includes (9) - %d\n", tree.Includes(9));
	printf("GetRoot - %d\n", tree.GetRoot());
	tree.Remove(9);
	printf("Includes (9) - %d\n", tree.Includes(9));
	printf("GetRoot - %d\n", tree.GetRoot());
	
	// Empty the list and check for emptiness
	tree.MakeEmpty();
	printf("IsEmpty - %d\n", tree.IsEmpty());
}
