#ifndef ASTRING_H
#define ASTRING_H

//=========================================================
// Interface Class: String
//
// What? Another string class implementation?  Well of course.
// This one is very small and straightforward.  It implements
// all the required assignment, construction, and comparison
// operators found in common usage.  The standard one that 
// comes with many compilers is often too big and bulky for
// simple usage.
//=========================================================

class String
{
public:
			String();
			String(const String& other);
			~String();

			// Specialized constructors
			String(const char);
			String(const unsigned long size);
			String(const char *initial);


		// Assignment operators
		String&	operator=(const String& other);
		String&	operator=(const char aChar);
		String&	operator=(const char *aCString);

		char &	operator[](unsigned long) const;
				operator const char * () const;

		String&	operator+=(const String& other);

		unsigned long	length() const;
		int		compare(const String&) const;

	static	String&		GetEmptyString();
	static	unsigned long 	CStringLength(const char *);
	static	bool			IsBlank(const char*);

protected:
	char *fBuffer;
	int	fBufferLength;

private:
};



//====================================
//  Comparison operators
//  For any class, there should be six
//  <, >, <=, >=, ==, !=
//====================================

inline int operator < (const String& left, const String& right)
{
	return left.compare(right) < 0;
}

inline int operator <= (const String& left, const String& right)
{
	return left.compare(right) <= 0;
}

inline int operator > (const String& left, const String& right)
{
	return left.compare(right) > 0;
}

inline int operator >= (const String& left, const String& right)
{
	return left.compare(right) >= 0;
}

inline int operator == (const String& left, const String& right)
{
	return left.compare(right) == 0;
}

inline int operator != (const String& left, const String& right)
{
	return left.compare(right) != 0;
}


//====================================
//  Simple math - Concatenation
//  +
//====================================
String operator+ (const String& left, const String& right);

#endif


