#ifndef BINARYTREE_H
#define BINARYTREE_H

#include "binarynode.h"
#include "treetraverse.h"

// Forward declarations
template <class T> class binaryTreeIterator;
template <class T> class binaryTree;

//==============================================================
// Template Interface: binaryTree<T>
//
// A binaryTree is a convenient base class for all binary tree 
// based structures, like searchTrees, splayTrees, and the like.
//
// The functions implemented here are minimal, but convenient.
//
// T - The type of values stored in the tree.  This type must
// implement the operator <, operator ==
//==============================================================
template <class T> 
class binaryTree
{
public:
				binaryTree():fRoot(0){};
				binaryTree(const T rootValue) {fRoot = new binaryNode<T>(rootValue);};
	virtual		~binaryTree(){FreeTree(fRoot);};

			const binaryTree<T> & operator = (const binaryTree<T> &rhs);
			
			binaryNode<T>	* GetRootNode() const { return fRoot;};			
			void	DeleteAll() {FreeTree(fRoot);fRoot =0;};
			bool	IsEmpty() const {return 0 == fRoot;};
			
protected:
	binaryNode<T> *	fRoot;
			void	FreeTree(binaryNode<T> *);
	
	friend class binaryTreeIterator<T>;
	
private:
	// No copy constructor
				binaryTree(const binaryTree<T> &);
};

//-------------------------------------------------------------
//
//-------------------------------------------------------------
template <class T>
const binaryTree<T> &
binaryTree<T>::operator=(const binaryTree<T> &rhs)
{
	if (this!= &rhs)
	{
		FreeTree(fRoot);
		if (rhs.fRoot)
			fRoot = rhs.fRoot->Copy();
	}
	
	return *this;
}

//-------------------------------------------------------------
//
//-------------------------------------------------------------
template <class T>
void
binaryTree<T>::FreeTree(binaryNode<T> *aTree)
{
	if (0 != aTree)
	{
		FreeTree((binaryNode<T> *)aTree->Left());
		FreeTree((binaryNode<T> *)aTree->Right());
		delete aTree;
	}
}


//==============================================================
// Template Interface: binaryTreeIterator<T>
//
// A simple iterator for a binaryTree.  You could simply use the
// root node of the search tree and use any of the pre,post,inorder
// traversers, but then you would be exposing the root node of the
// tree, which may not be desirable.
//
// This iterator will do an inorder iteration of the tree.
//==============================================================

template <class T>
class binaryTreeIterator : public inorderTraversal<T>
{
public:
				binaryTreeIterator(binaryTree<T> &);
				
	virtual bool	init();
	
protected:
	binaryTree<T>	& fTree;
	
private:
};

template <class T>
binaryTreeIterator<T>::binaryTreeIterator(binaryTree<T> &tree)
	: inorderTraversal<T>(tree.fRoot),
	fTree(tree)
{	
}

template <class T>
bool
binaryTreeIterator<T>::init()
{
	SetRoot(fTree.fRoot);
	return inorderTraversal<T>::init();
}


#endif
