
#include "bitvector.h"

bitVector::bitVector(unsigned int nElements)
	: fBitSequence((unsigned long)((nElements+7)/8),0)
{
}

bitVector::bitVector(const bitVector& other)
	: fBitSequence(other.fBitSequence)
{
}

unsigned long
bitVector::Length() const
{
	return 8*fBitSequence.Length();
}

// Bit changing operations
void
bitVector::Set(unsigned long idx)
{
	fBitSequence[ByteNumber(idx)] |= (unsigned char)Mask(idx);
}

void
bitVector::Clear(unsigned long idx)
{
	fBitSequence[ByteNumber(idx)] &= ~(unsigned char)Mask(idx);
}

void
bitVector::ClearAll()
{
	fBitSequence.SetAllToValue(0);
}

bool
bitVector::IsSet(unsigned long idx) const
{
	return (0 != (fBitSequence[ByteNumber(idx)] & Mask(idx)));
}

void
bitVector::Flip(unsigned long idx)
{
	fBitSequence[ByteNumber(idx)] ^= (unsigned char)Mask(idx);
}

// Set Operations
void
bitVector::Add(const bitVector& other)				// operator |=
{
	assert(Length() == other.Length());
	const unsigned long aLength = Length();
	
	for (unsigned long counter = 0; counter < aLength; counter++)
		fBitSequence[counter]|= other.fBitSequence[counter];
}

void
bitVector::Subtract(const bitVector& other)		// operator &= ~
{
	assert(Length() == other.Length());

	const unsigned long aLength = Length();
	for (unsigned long counter = 0; counter < aLength; counter++)
		fBitSequence[counter] &= ~other.fBitSequence[counter];
}

void
bitVector::IntersectWith(const bitVector& other)	//	operator &=
{
	assert(Length() == other.Length());
	const unsigned long aLength = Length();
	for (unsigned long counter = 0; counter < aLength; counter++)
		fBitSequence[counter] &= other.fBitSequence[counter];
}

bool
bitVector::Includes(const bitVector& other) const
{
	assert(Length() == other.Length());
	const unsigned long aLength = Length();
	for (unsigned long counter = 0; counter < aLength; counter++)
		if (other.fBitSequence[counter] != (fBitSequence[counter] & other.fBitSequence[counter]))
			return false;

	return true;
}
// Some utility routines
unsigned long
bitVector::ByteNumber(unsigned long idx) const
{
	return idx >> 3;
}
unsigned long
bitVector::Mask(unsigned long idx) const
{
	const unsigned long shiftAmount = idx & 07;	
	return 1 << shiftAmount;
}

