#ifndef HASHTABLE_H
#define HASHTABLE_H

#include "vector.h"

template <class B, class H, class T>
class hashTable
{
public:
				hashTable(unsigned long max, unsigned long (&f)(H&, const unsigned long tableSize));
				
	virtual bool	IsEmpty();
	virtual void	DeleteAll();
	
protected:
	friend class hashTableIterator<B, H, T>;
	
	const unsigned long fTableSize;
	vector<B> 			fBuckets;
	unsigned long 		(&fHashFunction)(H&, const unsigned long tableSize);
	
	unsigned long		Hash(const H& key) const;
	
	virtual iterator<T> * MakeIterator(unsigned long ) = 0;
	
private:
};

//===============================================================
// Implementation: hashTable
//===============================================================


template <class B, class H, class T>
bool
hashTable<B,H,T>::hashTable(unsigned long max, unsigned long (&f)(H&, const unsigned long tableSize))
	: fTableSize(max),
	fHashFunction(f)
{
}

template <class B, class H, class T>
bool
hashTable<B,H,T>::IsEmpty()
{
	int i;
	
	for (i=0; i< fTableSize; i++)
	{
		if (!fBuckets[i].IsEmpty())
			return false;
	}
	
	// If we've gottent to here, then all the buckets
	// are empty, therefore the whole hashTable is empty.
	return true;
}

template <class B, class H, class T>
void
hashTable<B,H,T>::DeleteAll()
{
	int i;
	
	for (i=0; i< fTableSize; i++)
	{
		fBuckets[i].DeleteAll())
	}
}

template <class B, class H, class T>
unsigned long
hashTable<B,H,T>::Hash(const H& key) const
{
	return (*fHashFunction)(key);
}


#endif
