#ifndef LISTITERATOR_H
#define LISTITERATOR_H

#include "iterator.h"
#include "list.h"

//===========================================
// Class Template: listIterator
// The concrete template class implementation of iterator
//===========================================
template<class AType> 
class listIterator : public iterator<AType>
{
public:
				listIterator(list<AType>& aList);

			// Implement the iterator protocol
			virtual	bool	init();
			virtual	long	next();
			virtual bool	done();
			
			virtual	AType	operator()();
			virtual	long	operator++();
			virtual	void	operator=(AType newValue);

			// Specifics to listIterator
			void	RemoveCurrent();
			void	AddBefore(AType newValue);
			void	AddAfter(AType newValue);

protected:
	link<AType> *	fCurrentLink;
	link<AType> *	fPreviousLink;
	list<AType> &	fList;

private:

};

//===========================================
// Class Template: listIterator
// Implementations
//===========================================

template <class AType>
listIterator<AType>::listIterator(list<AType>& aList)
	: fList(aList)
{
	init();
}

// Implement the iterator protocol
template <class AType>
bool
listIterator<AType>::init()
{
	fPreviousLink = 0;
	fCurrentLink = fList.fFirstLink;
	return fCurrentLink != 0;
}

template <class AType>
AType
listIterator<AType>::operator()()
{
	assert(fCurrentLink != 0);
	
	return fCurrentLink->fValue;
}

template <class AType>
void
listIterator<AType>::operator=(AType newValue)
{
	assert(fCurrentLink != 0);
	
	fCurrentLink->fValue = newValue;
}


template <class AType>
bool
listIterator<AType>::done()
{
	if (fCurrentLink == 0)
		if (fPreviousLink != 0)
			fCurrentLink = fPreviousLink->fNextLink;
	
	return fCurrentLink == 0;
}


template <class AType>
long
listIterator<AType>::operator ++()
{
	if (fCurrentLink == 0)
	{
		if (fPreviousLink == 0)
			fCurrentLink = fList.fFirstLink;
		else
			fCurrentLink = fPreviousLink->fNextLink;
	} else
	{
		fPreviousLink = fCurrentLink;
		fCurrentLink = fCurrentLink->fNextLink;
	}

	return fCurrentLink != 0;
}

template <class AType>
long
listIterator<AType>::next()
{
	if (fCurrentLink == 0)
	{
		if (fPreviousLink == 0)
			fCurrentLink = fList.fFirstLink;
		else
			fCurrentLink = fPreviousLink->fNextLink;
	} else
	{
		fPreviousLink = fCurrentLink;
		fCurrentLink = fCurrentLink->fNextLink;
	}

	return fCurrentLink != 0;
}


// list Iterators have these specifics
template <class AType>
void
listIterator<AType>::RemoveCurrent()
{
	assert(fCurrentLink != 0);
	
	if (fPreviousLink == 0)
		fList.fFirstLink = fCurrentLink->fNextLink;
	else
		fPreviousLink->fNextLink = fCurrentLink->fNextLink;
	
	delete fCurrentLink;
	
	fCurrentLink = 0;
}

template <class AType>
void
listIterator<AType>::AddBefore(AType newValue)
{
	if (fPreviousLink)
		fPreviousLink = fPreviousLink->Insert(newValue);
	else
	{
		fList.list<AType>::Add(newValue);
		fPreviousLink = fList.fFirstLink;
		fCurrentLink = fPreviousLink->fNextLink;
	}
}

template <class AType>
void
listIterator<AType>::AddAfter(AType newValue)
{
	if (fCurrentLink != 0)
	{
		fCurrentLink->Insert(newValue);
	} else if (fPreviousLink != 0)
		fCurrentLink = fPreviousLink->Insert(newValue);
	else
		fList.list<AType>::Add(newValue);
}


#endif
