#include "lock.h"
#include <SupportDefs.h>

long 
test_and_set(volatile long *addr)
{
	return atomic_or(addr, 1) ? 1 : 0 ;
}

/*
	Class: ASpinLock

	Implements a simple spin lock.	

	Simply instantiate one of these, then call Lock, and
	Unlock to acquire and release it.  This is a quick and
	dirty binary lock which is meant for locking of very short
	critical sections where spending a few clock cycles won't
	be a big deal.  If you want more long-term locking, then
	a semaphore would be more appropriate.
*/

ASpinLock::ASpinLock()
	: fLockStatus(0)
{
}

int
ASpinLock::Lock()
{
	while (test_and_set(&fLockStatus) == 1)
		;

	return 0;
}

int
ASpinLock::Unlock()
{
	// Release the lock by setting its value to
	// 0 atomically. Assuming this does a flush
	// of any caches so that other processors get
	// the word.
	//fLockStatus = 0;
	atomic_or(&fLockStatus,1);

	return 0;
}
