#ifndef LOCK_H
#define LOCK_H


extern long test_and_set(volatile long *addr);

/*
	Protocol: PBinaryLock
	
	Your basic locking protocol
*/

class PBinaryLock
{
public:
	virtual	int Lock() = 0;
	virtual	int Unlock() = 0;
};

/*
	Class: ASpinLock
	
	A spinlock is the simplest of the exclusive locking
	mechanisms.  The calling process simply spins in a 
	loop until the lock is released.  This is a good
	quick and dirty lock to be used on critical areas
	that are not going to be locked for very long.
	These areas are typically the instance variables
	that are used by other forms of locks.
	
	Spin locks are most effective when used whithin the
	kernel. This is typically in a device driver where you
	can disable interrupts while spinning so that you don't
	waste too much cpu when your thread is interrupted.
*/

class ASpinLock : public PBinaryLock
{
public:
		ASpinLock();
		
		int Lock();
		int Unlock();

protected:

private:		
	volatile long	fLockStatus;
};

#endif
