#include "portwatcher.h"
#include "port.h"
#include <malloc.h>
#include <stdio.h>


portWatcher::portWatcher(port *aPort, const char *aName, long aPriority)
	: thread(aName, aPriority),
	fPort(aPort),
	fTimeout(B_INFINITE_TIMEOUT),
	fMessageBuffer(0),
	fMessageBufferSize(0),
	fMessagesReceived(0),
	fOwnsPort(false)
{
	if (!aPort)
	{
		fPort = new port("pwatcherport");
		fOwnsPort = true;
	}
	
	fMessageBuffer = (char *)malloc(256);
	fMessageBufferSize = 256;
}

portWatcher::~portWatcher()
{
	Kill();
	int32 dummy;
	WaitForExit(&dummy);
	
	if (fMessageBuffer)
	{
		free(fMessageBuffer);
		fMessageBuffer=0;
	}
	fMessageBufferSize=0;

	if (fOwnsPort)
		delete fPort;
}

// We will loop here until the port is gone, or until 
// we are told to die.
status_t	
portWatcher::ThreadMain()
{
	if (!fPort)
		return B_ERROR;
	
	int32 aCode;
	ssize_t bufferSize = 0;
	status_t	error;
	
	while(1)
	{
		//
		// Don't need to worry about the B_INTERRUPTED error here. Code that
		// calls this should handle that situation.
		//
		bufferSize = fPort->CurrentBufferSizeEtc(B_TIMEOUT, fTimeout);
		if (bufferSize < 0)
			continue;

		if (bufferSize > fMessageBufferSize) 
		{
			fMessageBuffer = (char *)realloc(fMessageBuffer, bufferSize);
			fMessageBufferSize = bufferSize;
		}
		
		error = fPort->Read(&aCode, fMessageBuffer, bufferSize);

		if (error >= 0) 
		{
			fMessagesReceived++;
			HandleDataRead(aCode, fMessageBuffer, bufferSize);
		}
	}
}

status_t
portWatcher::AddFilter(portFilter *filter)
{
}

//=====================================================================
// Method: HandleDataRead
//
// Whenever a chunk of data has been read off the port, this method
// is called.  It is synchronous.  The thread will not pull off another
// message until we return from here.
//
// Typically, you will decode the message quickly and pass the data
// onto another thread which will actually do some processing with
// it.  In this default implementation, we don't do anything at all
// other than not that the message has been received.
//=====================================================================

void
portWatcher::HandleDataRead(const int32 code, void *data, ssize_t size)
{
	printf("HandleDataRead: Code: %d  Size: %d\n", code, size);
}
