#ifndef PORTWATCHER_H
#define PORTWATCHER_H

//=====================================================
// Class: portWatcher
//
// The purpose of the portWatcher class is to simply watch
// a port for incoming data.  You can compose higher level
// mechanisms utilizing portWatchers, ports, and threads.
//
// This default watcher will simply watch the port until
// the port goes away, or until it is killed.
// You may want to sub-class this class if you want to 
// do something special in the HandleDataRead method.
// This method is called whenever data has been received
// on the port.
//=====================================================

#include "thread.h"
#include <SupportDefs.h>



class port;
class portFilter;

class portWatcher : public thread
{
public:
			portWatcher(port *, const char *aName="portwatcher", long aPriority = B_NORMAL_PRIORITY);
	virtual ~portWatcher();
	
	// must implement from thread
	virtual	status_t	ThreadMain();
	
	// Methods for dealing with port message filtering
	virtual status_t	AddFilter(portFilter *filter);
	
	// Should implement if sub-classing
	virtual void	HandleDataRead(const int32 code, void *data, int32 size);
	
	int32	MessagesReceived(){return fMessagesReceived;};
	
protected:
	port		*fPort;
	bigtime_t	fTimeout;
	char		*fMessageBuffer;
	int32		fMessageBufferSize;
	int32		fMessagesReceived;
	bool		fOwnsPort;
	
private:
};

#endif
