#ifndef QUEUEVECTOR_H
#define QUEUEVECTOR_H

#include "queue.h"
#include "vector.h"

//========================================================
// Class: queueVector
//
// This is an implementation of the queue protocol using a
// vector as the underlying data structure for storage of
// the values.
//========================================================

template <class AType> class queueVector : public queue<AType>
{
public:
		queueVector(const unsigned long size);
		queueVector(const queueVector& other);
		
	virtual unsigned long	Capacity() const;	// How many can it hold
	virtual unsigned long	Size() const;		// How many currently held
	
	// Implementation of queue protocol
	virtual	void	MakeEmpty();
	virtual AType	Dequeue();
	virtual void	Enqueue(const AType val);
	virtual AType	Front();
	virtual bool	IsEmpty() const;
	

protected:
	vector<AType>		fData;		// The sequence that holds the data
	const unsigned long	fMaxSize;	// Max num items in queue
	unsigned long		fSize;		// How many items in queue
	unsigned long		fNextSlot;	// Where next item will be enqueued
	unsigned long		fFront;		// Where Front and Dequeue will get item from
};


//===================================================
// Implementation: queueVector
//===================================================

template <class AType> 
queueVector<AType>::queueVector(const unsigned long size)
	: fMaxSize(size),
	fData(size),
	fSize(0)
{
	// Initialize sequence to being empty
	DeleteAll();
}

template <class AType> 
queueVector<AType>::queueVector(const queueVector& other)
	: fMaxSize(other.fMaxSize),
	fData(other.fData),
	fNextSlot(other.fNextSlot),
	fFront(other.fFront),
	fSize(other.fSize)
{
}

template <class AType> 
void 
queueVector<AType>::MakeEmpty()
{
	fNextSlot = 0;
	fFront = 0;
	fSize = 0;
}

template <class AType> 
bool
queueVector<AType>::IsEmpty() const
{
	return (fNextSlot == fFront);
}


template <class AType>
unsigned long
queueVector<AType>::Capacity() const
{
	return fMaxSize;	
}

template <class AType> 
void
queueVector<AType>::Enqueue(const AType val)
{
	fData[fNextSlot] = val;
	fNextSlot++;
	if (fNextSlot >= fMaxSize)
		fNextSlot = 0;
		
	if (fSize < fMaxSize)
		fSize++;
}

template <class AType> 
AType
queueVector<AType>::Dequeue()
{
	if (IsEmpty())
		return fData[fFront];
		
	unsigned long dataloc = fFront;
	fFront++;
	if (fFront >= fMaxSize)
		fFront = 0;
	
	if (fSize > 0)
		fSize--;

	return fData[dataloc];
}

template <class AType> 
AType
queueVector<AType>::Front()
{
	return fData[fFront];
}

#endif
