#ifndef SIMPLELIST_H
#define SIMPLELIST_H

#include<assert.h>

// Forward declarations
#include "list.h"
#include "listiterator.h"
#include "doubleendedlist.h"

//======================================================
// template class: simplelist
//
// This class combines the base list class with some of 
// the more useful aspects of the listIterator template
// to make for a more useful list class.  The specific
// changes make it a bit easier to add and remove
// elements from the list.
//======================================================
template <class T> class simplelist : public doubleEndedList<T>
{
public:
				simplelist();
				simplelist(const list<T>& other);

	virtual void	AddBefore(T aValue, T before);
	virtual void	AddAfter(T aValue, T after);
	
	virtual void	Remove(T aValue);

protected:
	
private:
};

//===================================================
// Implementation: simplelist
//===================================================
template <class T>
simplelist<T>::simplelist()
	: doubleEndedList<T>()
{}

template <class T>
simplelist<T>::simplelist(const list<T>& other)
	: list<T>(other)
{
	if (other.IsEmpty())
	{
		fFirstLink = 0;
	} else
	{
		fFirstLink = other.fFirstLink->Copy();
	}
}


template <class T>
void
simplelist<T>::AddBefore(T aValue, T before)
{
	// use the iterator to find the right
	// spot in the list
	listIterator<T> iter(*this);

	for (iter.init(); !iter.done(); iter.next())
		if (iter() == before)
		{
			// Once we've found the right element,
			// use the iterator to remove it, we're done.
			iter.AddBefore(aValue);
			return ;
		}
}
template <class T>
void
simplelist<T>::AddAfter(T aValue, T after)
{
	// use the iterator to find the right
	// spot in the list
	listIterator<T> iter(*this);

	for (iter.init(); !iter.done(); iter.next())
		if (iter() == after)
		{
			// Once we've found the right element,
			// use the iterator to remove it, we're done.
			iter.AddAfter(aValue);
			return ;
		}
}


//
// To remove an element from the list, we first have to 
// find it.  Since we're template crazy, we use a iterator
// to find the element.  Then we use the iterator to remove
// the element since it already knows how to do it.
template <class T>
void
simplelist<T>::Remove(T aValue)
{
	// use the iterator to find the right
	// spot in the list
	listIterator<T> iter(*this);

	for (iter.init(); !iter.done(); iter.next())
		if (iter() == aValue)
		{
			// Once we've found the right element,
			// use the iterator to remove it, we're done.
			iter.RemoveCurrent();
			return ;
		}
}

#endif
