#ifndef SKEWHEAP_H
#define SKEWHEAP_H

#include "priorityqueue.h"
#include "binarynode.h"
#include <assert.h>

template <class T>
class skewHeap : public priorityQueue<T>
{
public:
			skewHeap();
			skewHeap(const skewHeap<T> &);
	virtual ~skewHeap();
	
	// priority queue protocol
	virtual void	Add(T value);
			void	Add(skewHeap<T> & other);
	
	virtual T		FindMin();
	virtual void	RemoveMin();
	
	virtual bool	IsEmpty();
	virtual void	MakeEmpty();
	
protected:
	binaryNode<T>	*fRoot;
	binaryNode<T>	* Merge(binaryNode<T> *, binaryNode<T> *);
		
private:
};


//-------------------------------------------------------------
//-------------------------------------------------------------
template <class T>
skewHeap<T>::skewHeap()
	: fRoot(0)
{
}

template <class T>
skewHeap<T>::skewHeap(const skewHeap<T> &rhs)
	: fRoot(0)
{
	if (rhs.fRoot)
		fRoot = rhs.fRoot->Copy();
}

template <class T>
skewHeap<T>::~skewHeap()
{
	if (fRoot)
		fRoot->Release();
	delete fRoot;
}

template <class T>
void
skewHeap<T>::Add(T value)
{
	binaryNode<T> *newNode = new binaryNode<T>(value);
	assert(newNode != 0);
	
	fRoot = Merge(fRoot, newNode);
}

template <class T>
void
skewHeap<T>::Add(skewHeap<T> & other)
{
	fRoot = Merge(fRoot, other.fRoot);
	other.fRoot = 0;
}

//-----------------------------------------------------
// Method: FindMin()
// This is the purpose of this structure, to return the 
// minimum value.  In this case, we just get it from the
// root binary node since that is where it is maintained.
//
//	T	-	Return of minimum value
//-----------------------------------------------------
template <class T>
T
skewHeap<T>::FindMin()
{
	return fRoot->fValue;
}

//-----------------------------------------------------
// Method: RemoveMin()
//
// Remove the minimum value from the heap, moving the
// next minimum to the top.  We do this by getting rid
// of the top, and merging the right and left to take
// its place.  The merge will figure out what should be
// first.
//-----------------------------------------------------

template <class T>
void
skewHeap<T>::RemoveMin()
{
	assert(!IsEmpty());
	binaryNode<T> *top = fRoot;
	fRoot = Merge(fRoot->fRight, fRoot->fLeft);
	delete top;
}

//-----------------------------------------------------
// Method: IsEmpty()
//
// Empty is determined by the existance of the fRoot node.
// if the node exists, then we are not empty.  If it doesn't
// then we are.  Simple enough.
//
// Return Value:
//	true - if empty
//  false - if not empty
//-----------------------------------------------------

template <class T>
bool
skewHeap<T>::IsEmpty()
{
	return fRoot == 0;
}

template <class T>
void
skewHeap<T>::MakeEmpty()
{
	fRoot->Release();
	delete fRoot;
	fRoot = 0;
}

template <class T>
binaryNode<T> *
skewHeap<T>::Merge(binaryNode<T> *heap1, binaryNode<T> *heap2)
{
	if (!heap1)
		return heap2;
		
	if (!heap2)
		return heap1;
	
	if (heap2->fValue < heap1->fValue)
		return Merge(heap2, heap1);
		
	binaryNode<T> * leftchild = heap1->fLeft;
	if (leftchild)
	{
		heap1->fLeft = Merge(heap1->fRight, heap2);
		heap1->fRight = leftchild;
	} else
		heap1->fLeft = heap2;
	
	return heap1;
}


#endif
